#!/usr/bin/python2.6
# vim:set encoding=utf8 et sw=4:

# Copyright © 2009,2010 Philipp Kern <pkern@debian.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from __future__ import with_statement

import psycopg2

from reportlab.pdfgen.canvas import Canvas
from reportlab.lib.pagesizes import A4
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import Image, Paragraph, Frame
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle

from optparse import OptionParser
from socket import gethostname
from tempfile import NamedTemporaryFile
import os.path
import sys

import cups
cups.require('1.9.39')

import getpass
from copy import copy


def init_fonts(path):
    pdfmetrics.registerFont(TTFont('Maiandra', os.path.join(path, 'Maian.ttf')))
    pdfmetrics.registerFont(TTFont('Maiandra Bold', os.path.join(path, 'Maiandb.ttf')))
    pdfmetrics.registerFont(TTFont('Maiandra Italic', os.path.join(path, 'Maiandit.ttf')))
    pdfmetrics.registerFontFamily('Maiandra', normal='Maiandra', bold='Maiandra Bold', italic='Maiandra Italic')

class NewsIter(object):
    """Iterator helper class used by News to ensure proper formatting of
    the news items returned by the database.  Takes the rows object
    as fetched from it and yield paragraphs generator-like for the
    headlines and paragraphs."""

    BODY_PLACEHOLDER = """\
        <p>Aus technischen Gründen kann dieser Newseintrag leider nicht
          abgedruckt werden.</p>
        <p>Bitte besuche unsere Homepage, um ihn zu lesen!</p>
        """

    def __init__(self, stylesheet, rows):
        self.stylesheet = stylesheet
        self.rows = rows
    def __iter__(self):
        for row in self.rows:
            try:
                yield Paragraph(row[0], self.stylesheet['Heading2'])
            except ValueError:
                # Skip this entry.
                continue

            style = self.stylesheet['BodyText']
            try:
                yield Paragraph(row[1], style)
            except:
                # Replace the body's text as it was not parseable by Platypus.
                yield Paragraph(NewsIter.BODY_PLACEHOLDER, style)

class News(object):
    """Singleton object that fetches News entries out of the database.
    Connection failures will result in an empty list of items.  The items
    itself are formatted as Platypus Paragraph objects, retrieveable
    through the items method."""

    def __init__(self, stylesheet):
        self.stylesheet = stylesheet
        if not 'conn' in self.__dict__:
            try:
                self._connect()
            except:
                self.conn = None

    def _connect(self):
        self.conn = psycopg2.connect("dbname='fsmi' user='prfproto' host='fsmi-db.fsmi.uni-karlsruhe.de' password='prfproto'")

    # Singleton
    _shared = {}
    def __new__(cls, *args, **kwargs):
        inst = object.__new__(cls)
        inst.__dict__ = cls._shared
        return inst

    def items(self):
        try:
            self._connect()
            cur = self.conn.cursor()
            cur.execute("select headline, content from published_news limit 5")
            return NewsIter(self.stylesheet, cur.fetchall())
        except:
            return iter([])

class CoverPage(object):
    """This CoverPage class is used to construct a two-sided cover page with
    a logo, a description of who and what it's intended to cover and a news
    entry list on its back.  The build method will yield a PDF in the
    target filename specified."""

    def __init__(self, who, what):
        self.who = who
        self.what = what

    @staticmethod
    def _center(text):
        return '<para alignment="center">' + text + '</para>'

    @staticmethod
    def _stylesheet():
        stylesheet = getSampleStyleSheet()
        stylesheet['Heading1'].fontName = 'Maiandra Bold'
        stylesheet['Heading1'].fontSize = 20

        stylesheet['Heading2'].fontName = 'Maiandra Bold'

        stylesheet.add(ParagraphStyle(name='FrontPageName',
                                      parent=stylesheet['Heading1'],
                                      fontSize=60,
                                      spaceBefore=10,
                                      spaceAfter=10,
                                      leading=60))
        return stylesheet

    def build(self, target):
        canvas = Canvas(target, pagesize=A4)
        width, height = A4
        canvas.setPageCompression(0)

        stylesheet = self._stylesheet()

        # News page comes first because the last page will appear on top.
        news_page = [
            Paragraph('<para alignment="center">Termine &amp; Aktuelles</para>', stylesheet['Heading1']),
            Paragraph('<para alignment="center">Immer aktuell unter<br /><font face="courier">http://www.fsmi.uni-karlsruhe.de</font>', stylesheet['Heading2']),
            ]
        news_page.extend(News(stylesheet).items())

        Frame(0, 0, width, height, 30, 30, 30, 30).addFromList(news_page, canvas)
        canvas.showPage()

        # Print owl.
        img = Image(os.path.join(BASE_PATH, 'FS-Eule.eps'))
        img.drawHeight = float(img.drawHeight) / img.drawWidth * (width / 2.0)
        img.drawWidth = width / 2.0
        img.hAlign = 'CENTER'
        img.drawOn(canvas, width / 4.0, height / 2.0)

        front_page = [
            Paragraph(self._center('<font face="Maiandra">Ausdruck für</font>'), stylesheet['Heading2']),
            Paragraph(self._center(self.who), stylesheet['FrontPageName']),
            Paragraph(self._center('(' + self.what + ')'), stylesheet['Heading1']),
            ]
        Frame(0, 0, width, height / 2.0 - 50, 20, 6, 20, 6).addFromList(front_page, canvas)
        canvas.showPage()

        canvas.save()

class PrintJob(object):
    """The PrintJob class is responsible for submitting a list of documents
    to the print spooler cups.  Some settings like the print job's name
    passed to cups are interface details parsed by the corresponding printer
    driver AffiDeluxeKlausur and should thus not be modified.  This class
    supports printing for both clients and for internal use."""

    def __init__(self):
        self.connection = cups.Connection()
        self.options = {
            'copies': '1',
            'multiple-document-handling': 'separate-documents-collated-copies',
            'Code': '2222',
            'fit-to-page': 'yes',
            'media': 'A4',
            'Staple': 'normal',
            }

    def _print(self, printer, options, filenames):
        # Please note that the job title is needed for AffiDeluxeKlausur
        # to ensure proper printing.  The value in parentheses is always
        # the number of files passed minus one, which is the cover page.
        # The printer driver will then switch to cover page mode after
        # n (= len - 1) documents.  The title does not matter for printing
        # through AffiDeluxeDuplex for internal use.
        self.connection.printFiles(
            printer=printer,
            filenames=filenames,
            title='Druckjob fuer %s (%d)' % (getpass.getuser(), len(filenames) - 1),
            options=options,
            )
        # XXX: If one of the target files does not exist, cups will silently
        # deschedule the job without raising an exception.  This should
        # either be checked or fixed in the future.

    def print_for_external_client(self, filenames):
        self._print('AffiDeluxeKlausur', self.options, filenames)

    def print_for_internal_client(self, uid, filenames):
        options = copy(self.options)
        options['Code'] = str(uid)
        self._print('AffiDeluxeDuplex', options, filenames)

BASE_PATH = os.path.dirname(__file__)

if gethostname()[0:2] == 'fs':
    FONT_PATH = '/usr/share/fonts/truetype/fsmi-fonts'
else:
    FONT_PATH = BASE_PATH
init_fonts(FONT_PATH)

def main(args):
    usage = """usage: %prog external "Name for cover" "Description for cover" file1 [fileN ...]\n"""\
            """       %prog internal acctno "Name for cover" "Description for cover" file1 [fileN ...]"""
    parser = OptionParser(usage=usage)
    parser.add_option('-d', '--debug', help='Enable debugging.',
                      action='store_true', dest='debug', default=False)
    (options, args) = parser.parse_args(args)

    args.reverse()
    selector = args.pop()
    if selector == 'external':
        who = args.pop()
        what = args.pop()

        with NamedTemporaryFile(delete=(not options.debug)) as cover:
            CoverPage(who, what).build(cover.name)
            if not options.debug:
                PrintJob().print_for_external_client(args + [cover.name])
            else:
                print cover.name
    elif selector == 'internal':
        uid = args.pop()
        who = args.pop()
        what = args.pop()

        with NamedTemporaryFile(delete=(not options.debug)) as cover:
            CoverPage(who, what).build(cover.name)
            if not options.debug:
                PrintJob().print_for_internal_client(uid, args + [cover.name])
            else:
                print cover.name
    else:
        print "Invalid selector: %s" % selector
        sys.exit(1)

if __name__ == '__main__':
    main(sys.argv[1:])

